﻿namespace PC3Player
{
    public partial class AdvancedSettingsForm : Form
    {
        public PlaylistConfig EditingObject { get; set; }
        public AdvancedSettingsForm(PlaylistConfig playlist)
        {
            InitializeComponent();
            EditingObject = playlist;

            PlaylistStartedDropDown.SelectedIndex = EditingObject.PlaylistStartOption;
            ShowEndedDropDown.SelectedIndex = EditingObject.PlaylistShowEndOption;
            IntermissionCheckBox.Checked = EditingObject.PlaylistIntermissionOption;
            IntermissionTimeCounter.Value = EditingObject.PlaylistIntermissionTimeOption;
            IntermissionUnitDropDown.SelectedIndex = EditingObject.PlaylistIntermissionUnitOption;
            LoopCheckBox.Checked = EditingObject.PlaylistLoopOption;

            ButtonPressDropDown.SelectedIndex = EditingObject.SelectorButtonPressOption;
            AlreadyPlayingDropDown.SelectedIndex = EditingObject.SelectorShowAlreadyPlayingOption;
            SelectedShowEndDropDown.SelectedIndex = EditingObject.SelectorShowEndOption;
            CooldownCheckBox.Checked = EditingObject.SelectorCooldownOption;
            CooldownTimeCounter.Value = EditingObject.SelectorCooldownTimeOption;
            CooldownUnitDropDown.SelectedIndex = EditingObject.SelectorCooldownUnitOption;
            SkitCheckBox.Checked = EditingObject.SelectorPlaySkitOption;

            FrameSkipOption.Value = EditingObject.FramesPerTick;
            FrameShiftOption.Value = EditingObject.FrameShift;
            ResyncTimeOption.Value = EditingObject.ResyncSeconds;

            IntermissionTimeCounter.Enabled = IntermissionCheckBox.Checked;
            IntermissionUnitDropDown.Enabled = IntermissionCheckBox.Checked;

            CooldownTimeCounter.Enabled = CooldownCheckBox.Checked;
            CooldownUnitDropDown.Enabled = CooldownCheckBox.Checked;

            int i = 0;
            foreach (ComboBox c in AssignmentPanel.Controls)
            {
                foreach (PlaylistShow s in EditingObject.Shows) c.Items.Add(s.Name);
                if (EditingObject.SelectorAssignments[i] == null) c.SelectedIndex = 0;
                else c.SelectedIndex = (int)EditingObject.SelectorAssignments[i] + 1;
                i++;
            }

            UpdateListView();
        }

        private void UpdateListView()
        {
            SkitListView.Items.Clear();
            foreach (PlaylistShow s in EditingObject.Skits) SkitListView.Items.Add(new ListViewItem(new[] { s.Name, s.Length }));
            SkitCheckBox.Enabled = EditingObject.Skits.Count != 0;
            if (EditingObject.Skits.Count == 0) SkitCheckBox.Checked = false;
        }

        private void IntermissionCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            IntermissionTimeCounter.Enabled = IntermissionCheckBox.Checked;
            IntermissionUnitDropDown.Enabled = IntermissionCheckBox.Checked;
        }

        private void CooldownCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            CooldownTimeCounter.Enabled = CooldownCheckBox.Checked;
            CooldownUnitDropDown.Enabled = CooldownCheckBox.Checked;
        }

        private void RemoveButton_Click(object sender, EventArgs e)
        {
            if (SkitListView.SelectedIndices.Count == 0) return;
            EditingObject.Skits.RemoveAt(SkitListView.SelectedIndices[0]);
            UpdateListView();
        }

        private void AddSkitButton_Click(object sender, EventArgs e)
        {
            if (!MainForm.ChannelMappingLoaded)
            {
                MessageBox.Show("You must load a channel mapping first.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            if (SelectShowFileDialog.ShowDialog() == DialogResult.OK)
            {
                string tempUSTData = File.ReadAllText(SelectShowFileDialog.FileName);
                if (!tempUSTData.StartsWith("UST,2,"))
                {
                    MessageBox.Show("Selected showtape is not a UST version 2 showtape.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                string[] headerData = tempUSTData.Split(';')[0].Split(',');
                string[] stringyBits = tempUSTData.Split(';')[1].Split(',');

                TimeSpan time = TimeSpan.FromSeconds(stringyBits.Length / 60);
                string formattedLength = time.ToString(@"hh\:mm\:ss");

                tempUSTData = null;
                stringyBits = null;

                if (headerData[3] != EditingObject.Mapping.Type)
                {
                    MessageBox.Show(
                        $"Selected showtape type does not match the loaded Channel Map." +
                        $"\nLoaded Channel Map Type: {EditingObject.Mapping.Type}" +
                        $"\nShowtape Type: {headerData[3]}"
                    , "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                EditingObject.Skits.Add(new PlaylistShow(headerData[2], formattedLength, headerData[3], SelectShowFileDialog.FileName));
                UpdateListView();
            }
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            EditingObject.PlaylistStartOption = PlaylistStartedDropDown.SelectedIndex;
            EditingObject.PlaylistShowEndOption = ShowEndedDropDown.SelectedIndex;
            EditingObject.PlaylistIntermissionOption = IntermissionCheckBox.Checked;
            EditingObject.PlaylistIntermissionTimeOption = (int)IntermissionTimeCounter.Value;
            EditingObject.PlaylistIntermissionUnitOption = IntermissionUnitDropDown.SelectedIndex;
            EditingObject.PlaylistLoopOption = LoopCheckBox.Checked;

            EditingObject.SelectorButtonPressOption = ButtonPressDropDown.SelectedIndex;
            EditingObject.SelectorShowAlreadyPlayingOption = AlreadyPlayingDropDown.SelectedIndex;
            EditingObject.SelectorShowEndOption = SelectedShowEndDropDown.SelectedIndex;
            EditingObject.SelectorCooldownOption = CooldownCheckBox.Checked;
            EditingObject.SelectorCooldownTimeOption = (int)CooldownTimeCounter.Value;
            EditingObject.SelectorCooldownUnitOption = CooldownUnitDropDown.SelectedIndex;
            EditingObject.SelectorPlaySkitOption = SkitCheckBox.Checked;

            EditingObject.ResyncSeconds = (int)ResyncTimeOption.Value;
            EditingObject.FrameShift = (int)FrameShiftOption.Value;
            EditingObject.FramesPerTick = (int)FrameSkipOption.Value;

            int i = 0;
            foreach (ComboBox c in AssignmentPanel.Controls)
            {
                if (c.SelectedIndex == 0) EditingObject.SelectorAssignments[i] = null;
                else EditingObject.SelectorAssignments[i] = c.SelectedIndex - 1;
                i++;
            }
        }
    }
}
